/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "AnalogueRepeaterPanel.h"

const unsigned int BORDER_SIZE   = 5U;
const unsigned int LABEL_WIDTH   = 75U;
const unsigned int DATA_WIDTH    = 100U;
const unsigned int LOGTEXT_WIDTH = 575U;

#include <wx/gbsizer.h>

CAnalogueRepeaterPanel::CAnalogueRepeaterPanel(wxWindow* parent, wxWindowID id) :
wxPanel(parent, id),
m_radioTransmit(NULL),
m_radioSquelch(NULL),
m_radioVOGAD(NULL),
m_toneburst(NULL),
m_ctcss(NULL),
m_radioDTMF(NULL),
m_extSquelch(NULL),
m_extTransmit(NULL),
m_extVOGAD(NULL),
m_extDTMF(NULL),
m_level(NULL),
m_state(NULL),
m_battery(NULL),
m_timeout(NULL),
m_lockout(NULL),
m_latch(NULL),
m_hang(NULL),
m_ack(NULL),
m_minimum(NULL),
m_callsign(NULL),
m_active(NULL),
m_logLine1(NULL),
m_logLine2(NULL),
m_logLine3(NULL)
{
	wxBoxSizer* panelSizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* status1Sizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, _("Status")), wxVERTICAL);
	wxFlexGridSizer* status2Sizer = new wxFlexGridSizer(6);

	wxStaticText* stateLabel = new wxStaticText(this, -1, _("State:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	status2Sizer->Add(stateLabel, 0, wxALL, BORDER_SIZE);

	m_state = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	status2Sizer->Add(m_state, 0, wxALL, BORDER_SIZE);

	wxStaticText* levelLabel = new wxStaticText(this, -1, _("Level:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	status2Sizer->Add(levelLabel, 0, wxALL, BORDER_SIZE);

	m_level = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	status2Sizer->Add(m_level, 0, wxALL, BORDER_SIZE);

	wxStaticText* batteryLabel = new wxStaticText(this, -1, _("Battery"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	status2Sizer->Add(batteryLabel, 0, wxALL, BORDER_SIZE);

	m_battery = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	status2Sizer->Add(m_battery, 0, wxALL, BORDER_SIZE);

	status1Sizer->Add(status2Sizer);
	panelSizer->Add(status1Sizer, 0, wxALL, BORDER_SIZE);

	wxStaticBoxSizer* radio1Sizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, _("Radio")), wxVERTICAL);
	wxFlexGridSizer* radio2Sizer = new wxFlexGridSizer(6);

	wxStaticText* radioSquelchLabel = new wxStaticText(this, -1, _("Squelch:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	radio2Sizer->Add(radioSquelchLabel, 0, wxALL, BORDER_SIZE);

	m_radioSquelch = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	radio2Sizer->Add(m_radioSquelch, 0, wxALL, BORDER_SIZE);

	wxStaticText* radioTransmitLabel = new wxStaticText(this, -1, _("Transmit:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	radio2Sizer->Add(radioTransmitLabel, 0, wxALL, BORDER_SIZE);

	m_radioTransmit = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	radio2Sizer->Add(m_radioTransmit, 0, wxALL, BORDER_SIZE);

	wxStaticText* radioVOGADLabel = new wxStaticText(this, -1, _("VOGAD:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	radio2Sizer->Add(radioVOGADLabel, 0, wxALL, BORDER_SIZE);

	m_radioVOGAD = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	radio2Sizer->Add(m_radioVOGAD, 0, wxALL, BORDER_SIZE);

	wxStaticText* toneburstLabel = new wxStaticText(this, -1, _("1750Hz:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	radio2Sizer->Add(toneburstLabel, 0, wxALL, BORDER_SIZE);

	m_toneburst = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	radio2Sizer->Add(m_toneburst, 0, wxALL, BORDER_SIZE);

	wxStaticText* ctcssLabel = new wxStaticText(this, -1, _("CTCSS:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	radio2Sizer->Add(ctcssLabel, 0, wxALL, BORDER_SIZE);

	m_ctcss = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	radio2Sizer->Add(m_ctcss, 0, wxALL, BORDER_SIZE);

	wxStaticText* radioDTMFLabel = new wxStaticText(this, -1, _("DTMF:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	radio2Sizer->Add(radioDTMFLabel, 0, wxALL, BORDER_SIZE);

	m_radioDTMF = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	radio2Sizer->Add(m_radioDTMF, 0, wxALL, BORDER_SIZE);

	radio1Sizer->Add(radio2Sizer);
	panelSizer->Add(radio1Sizer, 0, wxALL, BORDER_SIZE);

	wxStaticBoxSizer* external1Sizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, _("External")), wxVERTICAL);
	wxFlexGridSizer* external2Sizer = new wxFlexGridSizer(6);

	wxStaticText* extSquelchLabel = new wxStaticText(this, -1, _("Squelch:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	external2Sizer->Add(extSquelchLabel, 0, wxALL, BORDER_SIZE);

	m_extSquelch = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	external2Sizer->Add(m_extSquelch, 0, wxALL, BORDER_SIZE);

	wxStaticText* extTransmitLabel = new wxStaticText(this, -1, _("Transmit:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	external2Sizer->Add(extTransmitLabel, 0, wxALL, BORDER_SIZE);

	m_extTransmit = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	external2Sizer->Add(m_extTransmit, 0, wxALL, BORDER_SIZE);

	wxStaticText* extVOGADLabel = new wxStaticText(this, -1, _("VOGAD:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	external2Sizer->Add(extVOGADLabel, 0, wxALL, BORDER_SIZE);

	m_extVOGAD = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	external2Sizer->Add(m_extVOGAD, 0, wxALL, BORDER_SIZE);

	wxStaticText* extDTMFLabel = new wxStaticText(this, -1, _("DTMF:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	external2Sizer->Add(extDTMFLabel, 0, wxALL, BORDER_SIZE);

	m_extDTMF = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	external2Sizer->Add(m_extDTMF, 0, wxALL, BORDER_SIZE);

	external1Sizer->Add(external2Sizer);
	panelSizer->Add(external1Sizer, 0, wxALL, BORDER_SIZE);

	wxStaticBoxSizer* timers1Sizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, _("Timers")), wxVERTICAL);
	wxFlexGridSizer* timers2Sizer = new wxFlexGridSizer(6);

	wxStaticText* timeoutLabel = new wxStaticText(this, -1, _("Timeout:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	timers2Sizer->Add(timeoutLabel, 0, wxALL, BORDER_SIZE);

	m_timeout = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	timers2Sizer->Add(m_timeout, 0, wxALL, BORDER_SIZE);

	wxStaticText* lockoutLabel = new wxStaticText(this, -1, _("Lockout:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	timers2Sizer->Add(lockoutLabel, 0, wxALL, BORDER_SIZE);

	m_lockout = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	timers2Sizer->Add(m_lockout, 0, wxALL, BORDER_SIZE);

	wxStaticText* latchLabel = new wxStaticText(this, -1, _("Latch:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	timers2Sizer->Add(latchLabel, 0, wxALL, BORDER_SIZE);

	m_latch = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	timers2Sizer->Add(m_latch, 0, wxALL, BORDER_SIZE);

	wxStaticText* hangLabel = new wxStaticText(this, -1, _("Hang:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	timers2Sizer->Add(hangLabel, 0, wxALL, BORDER_SIZE);

	m_hang = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	timers2Sizer->Add(m_hang, 0, wxALL, BORDER_SIZE);

	wxStaticText* ackLabel = new wxStaticText(this, -1, _("Ack:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	timers2Sizer->Add(ackLabel, 0, wxALL, BORDER_SIZE);

	m_ack = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	timers2Sizer->Add(m_ack, 0, wxALL, BORDER_SIZE);

	wxStaticText* minimumLabel = new wxStaticText(this, -1, _("Minimum:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	timers2Sizer->Add(minimumLabel, 0, wxALL, BORDER_SIZE);

	m_minimum = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	timers2Sizer->Add(m_minimum, 0, wxALL, BORDER_SIZE);

	wxStaticText* callsignLabel = new wxStaticText(this, -1, _("Callsign:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	timers2Sizer->Add(callsignLabel, 0, wxALL, BORDER_SIZE);

	m_callsign = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	timers2Sizer->Add(m_callsign, 0, wxALL, BORDER_SIZE);

	wxStaticText* activeLabel = new wxStaticText(this, -1, _("Active:"), wxDefaultPosition, wxSize(LABEL_WIDTH, -1));
	timers2Sizer->Add(activeLabel, 0, wxALL, BORDER_SIZE);

	m_active = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(DATA_WIDTH, -1));
	timers2Sizer->Add(m_active, 0, wxALL, BORDER_SIZE);

	timers1Sizer->Add(timers2Sizer);
	panelSizer->Add(timers1Sizer, 0, wxALL, BORDER_SIZE);

	wxStaticBoxSizer* log1Sizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, _("Log")), wxVERTICAL);
	wxBoxSizer* log2Sizer = new wxBoxSizer(wxVERTICAL);

	m_logLine1 = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(LOGTEXT_WIDTH, -1));
	m_logLine1->Wrap(LOGTEXT_WIDTH);
	log2Sizer->Add(m_logLine1, 0, wxTOP | wxLEFT | wxRIGHT, BORDER_SIZE);

	m_logLine2 = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(LOGTEXT_WIDTH, -1));
	m_logLine2->Wrap(LOGTEXT_WIDTH);
	log2Sizer->Add(m_logLine2, 0, wxLEFT | wxRIGHT, BORDER_SIZE);

	m_logLine3 = new wxStaticText(this, -1, wxEmptyString, wxDefaultPosition, wxSize(LOGTEXT_WIDTH, -1));
	m_logLine3->Wrap(LOGTEXT_WIDTH);
	log2Sizer->Add(m_logLine3, 0, wxLEFT | wxRIGHT | wxBOTTOM, BORDER_SIZE);

	log1Sizer->Add(log2Sizer);
	panelSizer->Add(log1Sizer, 0, wxALL, BORDER_SIZE);

	SetSizer(panelSizer);
	panelSizer->SetSizeHints(this);
}

CAnalogueRepeaterPanel::~CAnalogueRepeaterPanel()
{
}

void CAnalogueRepeaterPanel::update(const CAnalogueRepeaterStatusData& status)
{
	bool radioTransmit = status.isRadioTransmit();
	m_radioTransmit->SetLabel(radioTransmit ? _("On") : _("Off"));

	switch (status.getSquelch()) {
		case AS_CLOSED:
		case AS_EXTERNAL:
			m_radioSquelch->SetLabel(_("Closed"));
			break;
		default:
			m_radioSquelch->SetLabel(_("Open"));
			break;
	}

	bool battery = status.isBattery();
	m_battery->SetLabel(battery ? _("Yes") : _("No"));

	wxString buffer;
	wxFloat32 level = status.getLevel();
	if (level == 0.0F)
		level = 1.0F;
	buffer.Printf(wxT("%.1f dB"), 20.0F * ::log10(level));
	m_level->SetLabel(buffer);

	wxFloat32 vogad = status.getRadioVOGAD();
	if (vogad == 0.0F)
		vogad = 1.0F;
	buffer.Printf(wxT("%.1f dB"), 20.0F * ::log10(vogad));
	m_radioVOGAD->SetLabel(buffer);

	vogad = status.getExternalVOGAD();
	if (vogad == 0.0F)
		vogad = 1.0F;
	buffer.Printf(wxT("%.1f dB"), 20.0F * ::log10(vogad));
	m_extVOGAD->SetLabel(buffer);

	wxString dtmf = status.getRadioDTMF();
	m_radioDTMF->SetLabel(dtmf);

	dtmf = status.getExternalDTMF();
	m_extDTMF->SetLabel(dtmf);

	ANALOGUE_RPT_STATE state = status.getState();
	switch (state) {
		case ARS_SHUTDOWN:
			m_state->SetLabel(_("Shutdown"));
			break;
		case ARS_LISTENING:
			m_state->SetLabel(_("Listening"));
			break;
		case ARS_RELAYING_RADIO:
			m_state->SetLabel(_("Relaying Radio"));
			break;
		case ARS_RELAYING_EXTERNAL:
			m_state->SetLabel(_("Relaying External"));
			break;
		case ARS_WAITING_RADIO:
		case ARS_WAITING_EXTERNAL:
			m_state->SetLabel(_("Waiting"));
			break;
		case ARS_TIMEOUT_RADIO:
		case ARS_TIMEOUT_EXTERNAL:
			m_state->SetLabel(_("Timeout"));
			break;
		case ARS_LOCKOUT_RADIO:
		case ARS_LOCKOUT_EXTERNAL:
			m_state->SetLabel(_("Lockout"));
			break;
	}

	bool toneburst = status.getSquelch() == AS_TONEBURST;
	m_toneburst->SetLabel(toneburst ? _("Yes") : _("No"));

	bool ctcss = status.getSquelch() == AS_CTCSS;
	m_ctcss->SetLabel(ctcss ? _("Yes") : _("No"));

	buffer.Printf(wxT("%u/%u"), status.getTimeoutTimer(), status.getTimeoutExpiry());
	m_timeout->SetLabel(buffer);

	buffer.Printf(wxT("%u/%u"), status.getLockoutTimer(), status.getLockoutExpiry());
	m_lockout->SetLabel(buffer);

	buffer.Printf(wxT("%u/%u"), status.getHangTimer(), status.getHangExpiry());
	m_hang->SetLabel(buffer);

	buffer.Printf(wxT("%u/%u"), status.getLatchTimer(), status.getLatchExpiry());
	m_latch->SetLabel(buffer);

	buffer.Printf(wxT("%u/%u"), status.getAckTimer(), status.getAckExpiry());
	m_ack->SetLabel(buffer);

	buffer.Printf(wxT("%u/%u"), status.getMinimumTimer(), status.getMinimumExpiry());
	m_minimum->SetLabel(buffer);

	buffer.Printf(wxT("%u/%u"), status.getCallsignTimer(), status.getCallsignExpiry());
	m_callsign->SetLabel(buffer);

	buffer.Printf(wxT("%u/%u"), status.getActiveTimer(), status.getActiveExpiry());
	m_active->SetLabel(buffer);

	bool extSquelch = status.getSquelch() == AS_EXTERNAL;
	m_extSquelch->SetLabel(extSquelch ? _("Open") : _("Closed"));

	bool extTransmit = status.isExternalTransmit();
	m_extTransmit->SetLabel(extTransmit ? _("On") : _("Off"));
}

void CAnalogueRepeaterPanel::showLog(const wxString& text)
{
	m_logLine3->SetLabel(m_logLine2->GetLabel());
	m_logLine2->SetLabel(m_logLine1->GetLabel());
	m_logLine1->SetLabel(text);
}
