/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "AnalogueRepeaterConfigFrame.h"
#include "AnalogueRepeaterConfigApp.h"
#include "Version.h"

const unsigned int BORDER_SIZE  = 5U;

#include <wx/gbsizer.h>
#include <wx/notebook.h>
#include <wx/aboutdlg.h>

#include <cmath>

BEGIN_EVENT_TABLE(CAnalogueRepeaterConfigFrame, wxFrame)
	EVT_MENU(wxID_EXIT, CAnalogueRepeaterConfigFrame::onQuit)
	EVT_MENU(wxID_ABOUT, CAnalogueRepeaterConfigFrame::onAbout)

	EVT_CLOSE(CAnalogueRepeaterConfigFrame::onClose)
END_EVENT_TABLE()

CAnalogueRepeaterConfigFrame::CAnalogueRepeaterConfigFrame(const wxString& title) :
wxFrame(NULL, -1, title),
m_callsign(NULL),
m_ack(NULL),
m_times(NULL),
m_tones(NULL),
m_feel(NULL),
m_radio(NULL),
m_external(NULL),
m_controller(NULL),
m_dtmf1(NULL),
m_dtmf2(NULL),
m_active(NULL)
{
	SetMenuBar(createMenuBar());

	wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);

	wxPanel* panel = new wxPanel(this);

	wxBoxSizer* panelSizer = new wxBoxSizer(wxVERTICAL);

	wxString callsign, beacon;
	unsigned int idSpeed, idFreq;
	wxFloat32 idLevel1, idLevel2;
	::wxGetApp().getCallsign(callsign, beacon, idSpeed, idFreq, idLevel1, idLevel2);

	wxString ackRadio, ackExternal;
	unsigned int ackSpeed, ackFreq, ackDelay, ackMinimum;
	wxFloat32 ackLevel;
	::wxGetApp().getAck(ackRadio, ackExternal, ackSpeed, ackFreq, ackLevel, ackDelay, ackMinimum);

	unsigned int callsignTime, timeout, lockoutTime, hangTime, latchTime;
	::wxGetApp().getTimes(callsignTime, timeout, lockoutTime, hangTime, latchTime);

	bool tbEnable, ctcssInternal;
	wxFloat32 tbThreshold, ctcssFreq, ctcssThresh, ctcssLevel;
	ANALOGUE_CTCSS_OUTPUT ctcssOutput;
	::wxGetApp().getTones(tbEnable, tbThreshold, ctcssFreq, ctcssInternal, ctcssThresh, ctcssLevel, ctcssOutput);

	ANALOGUE_CALLSIGN_START callsignAtStart;
	ANALOGUE_TIMEOUT_TYPE timeoutType;
	bool callsignAtEnd;
	ANALOGUE_CALLSIGN_HOLDOFF callsignHoldoff;
	::wxGetApp().getFeel(callsignAtStart, callsignAtEnd, timeoutType, callsignHoldoff);

	wxString radioReadDevice, radioWriteDevice;
	unsigned int radioAudioDelay;
	bool radioDeEmphasis, radioPreEmphasis, radioVOGAD;
	::wxGetApp().getRadio(radioReadDevice, radioWriteDevice, radioAudioDelay, radioDeEmphasis, radioPreEmphasis, radioVOGAD);

	ANALOGUE_EXTERNAL_MODE externalMode;
	bool externalVOGAD;
	wxString externalReadDevice, externalWriteDevice, externalInterfaceDevice;
	SERIALPIN externalInterfaceTXPin, externalInterfaceRXPin;
	::wxGetApp().getExternal(externalMode, externalReadDevice, externalWriteDevice, externalVOGAD, externalInterfaceDevice, externalInterfaceTXPin, externalInterfaceRXPin);

	wxString interfaceType;
	unsigned int pttDelay;
	bool pttInvert, squelchInvert;
	::wxGetApp().getController(interfaceType, pttDelay, pttInvert, squelchInvert);

	bool dtmfRadio, dtmfExternal;
	wxString dtmfShutdown, dtmfStartup, dtmfTimeout, dtmfTimeReset, dtmfOutput1, dtmfOutput2, dtmfOutput3, dtmfOutput4;
	wxString dtmfCommand1, dtmfCommand1Line, dtmfCommand2, dtmfCommand2Line;
	wxFloat32 dtmfThreshold;
	::wxGetApp().getDTMF(dtmfRadio, dtmfExternal, dtmfShutdown, dtmfStartup, dtmfTimeout, dtmfTimeReset, dtmfCommand1, dtmfCommand1Line, dtmfCommand2, dtmfCommand2Line, dtmfOutput1, dtmfOutput2, dtmfOutput3, dtmfOutput4, dtmfThreshold);

	unsigned int activeHangTime;
	::wxGetApp().getActiveHang(activeHangTime);

	wxNotebook* noteBook = new wxNotebook(panel, -1);

	m_callsign = new CAnalogueRepeaterConfigCallsignSet(noteBook, -1, APPLICATION_NAME, callsign, beacon, idSpeed, idFreq, idLevel1, idLevel2);
	noteBook->AddPage(m_callsign, _("Callsign"), true);

	m_ack = new CAnalogueRepeaterConfigAckSet(noteBook, -1, APPLICATION_NAME, ackRadio, ackExternal, ackSpeed, ackFreq, ackLevel, ackDelay, ackMinimum);
	noteBook->AddPage(m_ack, _("Acknowldegements"), false);

	m_times = new CAnalogueRepeaterConfigTimesSet(noteBook, -1, APPLICATION_NAME, callsignTime, timeout, lockoutTime, hangTime, latchTime);
	noteBook->AddPage(m_times, _("Timers"), false);

	m_tones = new CAnalogueRepeaterConfigTonesSet(noteBook, -1, APPLICATION_NAME, tbEnable, tbThreshold, ctcssFreq, ctcssInternal, ctcssThresh, ctcssLevel, ctcssOutput);
	noteBook->AddPage(m_tones, _("Access Tones"), false);

	m_feel = new CAnalogueRepeaterConfigFeelSet(noteBook, -1, APPLICATION_NAME, callsignAtStart, callsignAtEnd, timeoutType, callsignHoldoff);
	noteBook->AddPage(m_feel, _("Feel"), false);

	m_radio = new CAnalogueRepeaterConfigRadioSet(noteBook, -1, APPLICATION_NAME, radioReadDevice, radioWriteDevice, radioAudioDelay, radioDeEmphasis, radioPreEmphasis, radioVOGAD);
	noteBook->AddPage(m_radio, _("Radio"), false);

	m_external = new CAnalogueRepeaterConfigExternalSet(noteBook, -1, APPLICATION_NAME, externalMode, externalReadDevice, externalWriteDevice, externalVOGAD, externalInterfaceDevice, externalInterfaceTXPin, externalInterfaceRXPin);
	noteBook->AddPage(m_external, _("External"), false);

	m_controller = new CAnalogueRepeaterConfigControllerSet(noteBook, -1, APPLICATION_NAME, interfaceType, pttDelay, pttInvert, squelchInvert);
	noteBook->AddPage(m_controller, _("Controller"), false);

	m_dtmf1 = new CAnalogueRepeaterConfigDTMF1Set(noteBook, -1, APPLICATION_NAME, dtmfRadio, dtmfExternal, dtmfShutdown, dtmfStartup, dtmfTimeout, dtmfTimeReset, dtmfOutput1, dtmfOutput2, dtmfOutput3, dtmfOutput4, dtmfThreshold);
	noteBook->AddPage(m_dtmf1, _("DTMF Commands 1"), false);

	m_dtmf2 = new CAnalogueRepeaterConfigDTMF2Set(noteBook, -1, APPLICATION_NAME, dtmfCommand1, dtmfCommand1Line, dtmfCommand2, dtmfCommand2Line);
	noteBook->AddPage(m_dtmf2, _("DTMF Commands 2"), false);

	m_active = new CActiveHangSet(noteBook, -1, APPLICATION_NAME, activeHangTime);
	noteBook->AddPage(m_active, _("Active Output"), false);

	panelSizer->Add(noteBook, 1, wxALL | wxGROW, BORDER_SIZE);

	panel->SetSizer(panelSizer);
	panelSizer->SetSizeHints(panel);

	mainSizer->Add(panel);

	SetSizer(mainSizer);
	mainSizer->SetSizeHints(this);
}

CAnalogueRepeaterConfigFrame::~CAnalogueRepeaterConfigFrame()
{
}

wxMenuBar* CAnalogueRepeaterConfigFrame::createMenuBar()
{
	wxMenu* fileMenu = new wxMenu();

	fileMenu->Append(wxID_EXIT, _("Exit"));

	wxMenu* helpMenu = new wxMenu();
	helpMenu->Append(wxID_ABOUT, _("About Analogue Repeater Config"));

	wxMenuBar* menuBar = new wxMenuBar();
	menuBar->Append(fileMenu, _("File"));
	menuBar->Append(helpMenu, _("Help"));

	return menuBar;
}

void CAnalogueRepeaterConfigFrame::onQuit(wxCommandEvent& event)
{
	Close(true);
}

void CAnalogueRepeaterConfigFrame::onClose(wxCloseEvent& event)
{
	wxMessageDialog dialog(this, _("Save changes?"), _("Analogue Repeater Config"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION);
	int id = dialog.ShowModal();

	if (id == wxID_NO)
		Destroy();

	if (!m_callsign->Validate() || !m_ack->Validate() || !m_times->Validate() || !m_tones->Validate() || !m_feel->Validate() || !m_radio->Validate() || !m_external->Validate() || !m_controller->Validate() || !m_dtmf1->Validate() || !m_dtmf2->Validate()) {
		event.Veto(true);
		return;
	}

	saveConfig();

	Destroy();
}

void CAnalogueRepeaterConfigFrame::saveConfig()
{
	wxString callsign    = m_callsign->getCallsign();
	wxString beacon      = m_callsign->getBeacon();
	unsigned int idSpeed = m_callsign->getSpeed();
	unsigned int idFreq  = m_callsign->getFreq();
	wxFloat32 idLevel1   = m_callsign->getLevel1();
	wxFloat32 idLevel2   = m_callsign->getLevel2();
	::wxGetApp().setCallsign(callsign, beacon, idSpeed, idFreq, idLevel1, idLevel2);

	wxString ackRadio       = m_ack->getRadioAck();
	wxString ackExternal    = m_ack->getExternalAck();
	unsigned int ackSpeed   = m_ack->getSpeed();
	unsigned int ackFreq    = m_ack->getFreq();
	wxFloat32 ackLevel      = m_ack->getLevel();
	unsigned int ackDelay   = m_ack->getDelay();
	unsigned int ackMinimum = m_ack->getMinimum();
	::wxGetApp().setAck(ackRadio, ackExternal, ackSpeed, ackFreq, ackLevel, ackDelay, ackMinimum);

	unsigned int callsignTime = m_times->getCallsignTime();
	unsigned int timeout      = m_times->getTimeout();
	unsigned int lockoutTime  = m_times->getLockoutTime();
	unsigned int hangTime     = m_times->getHangTime();
	unsigned int latchTime    = m_times->getLatchTime();
	::wxGetApp().setTimes(callsignTime, timeout, lockoutTime, hangTime, latchTime);

	bool tbEnable                     = m_tones->getTbEnable();
	wxFloat32 tbThreshold             = m_tones->getTbThreshold();
	bool ctcssInternal                = m_tones->getCTCSSInternal();
	wxFloat32 ctcssFreq               = m_tones->getCTCSSFreq();
	wxFloat32 ctcssThresh             = m_tones->getCTCSSThresh();
	wxFloat32 ctcssLevel              = m_tones->getCTCSSLevel();
	ANALOGUE_CTCSS_OUTPUT ctcssOutput = m_tones->getCTCSSOutput();
	::wxGetApp().setTones(tbEnable, tbThreshold, ctcssFreq, ctcssInternal, ctcssThresh, ctcssLevel, ctcssOutput);

	ANALOGUE_CALLSIGN_START callsignAtStart   = m_feel->getCallAtStart();
	bool callsignAtEnd                        = m_feel->getCallAtEnd();
	ANALOGUE_TIMEOUT_TYPE timeoutType         = m_feel->getTimeoutType();
	ANALOGUE_CALLSIGN_HOLDOFF callsignHoldoff = m_feel->getHoldoff();
	::wxGetApp().setFeel(callsignAtStart, callsignAtEnd, timeoutType, callsignHoldoff);

	wxString radioReadDevice     = m_radio->getReadDevice();
	wxString radioWriteDevice    = m_radio->getWriteDevice();
	unsigned int radioAudioDelay = m_radio->getAudioDelay();
	bool radioDeEmphasis         = m_radio->getDeEmphasis();
	bool radioPreEmphasis        = m_radio->getPreEmphasis();
	bool radioVOGAD              = m_radio->getVOGAD();
	::wxGetApp().setRadio(radioReadDevice, radioWriteDevice, radioAudioDelay, radioDeEmphasis, radioPreEmphasis, radioVOGAD);

	ANALOGUE_EXTERNAL_MODE externalMode = m_external->getMode();
	wxString externalReadDevice         = m_external->getReadDevice();
	wxString externalWriteDevice        = m_external->getWriteDevice();
	bool externalVOGAD                  = m_external->getVOGAD();
	wxString externalInterfaceDevice    = m_external->getDevice();
	SERIALPIN externalInterfaceTXPin    = m_external->getTXPin();
	SERIALPIN externalInterfaceRXPin    = m_external->getRXPin();
	::wxGetApp().setExternal(externalMode, externalReadDevice, externalWriteDevice, externalVOGAD, externalInterfaceDevice, externalInterfaceTXPin, externalInterfaceRXPin);

	wxString interfaceType = m_controller->getType();
	unsigned int pttDelay  = m_controller->getPTTDelay();
	bool pttInvert         = m_controller->getPTTInvert();
	bool squelchInvert     = m_controller->getSquelchInvert();
	::wxGetApp().setController(interfaceType, pttDelay, pttInvert, squelchInvert);

	bool dtmfRadio            = m_dtmf1->getRadio();
	bool dtmfExternal         = m_dtmf1->getExternal();
	wxString dtmfShutdown     = m_dtmf1->getShutdown();
	wxString dtmfStartup      = m_dtmf1->getStartup();
	wxString dtmfTimeout      = m_dtmf1->getTimeout();
	wxString dtmfTimeReset    = m_dtmf1->getTimeReset();
	wxString dtmfCommand1     = m_dtmf2->getCommand1();
	wxString dtmfCommand1Line = m_dtmf2->getCommand1Line();
	wxString dtmfCommand2     = m_dtmf2->getCommand2();
	wxString dtmfCommand2Line = m_dtmf2->getCommand2Line();
	wxString dtmfOutput1      = m_dtmf1->getOutput1();
	wxString dtmfOutput2      = m_dtmf1->getOutput2();
	wxString dtmfOutput3      = m_dtmf1->getOutput3();
	wxString dtmfOutput4      = m_dtmf1->getOutput4();
	wxFloat32 dtmfThreshold   = m_dtmf1->getThreshold();
	::wxGetApp().setDTMF(dtmfRadio, dtmfExternal, dtmfShutdown, dtmfStartup, dtmfTimeout, dtmfTimeReset, dtmfCommand1, dtmfCommand1Line, dtmfCommand2, dtmfCommand2Line, dtmfOutput1, dtmfOutput2, dtmfOutput3, dtmfOutput4, dtmfThreshold);

	unsigned int activeHangTime = m_active->getTime();
	::wxGetApp().setActiveHang(activeHangTime);
}

void CAnalogueRepeaterConfigFrame::onAbout(wxCommandEvent& event)
{
	wxAboutDialogInfo info;
	info.AddDeveloper(wxT("Jonathan Naylor, G4KLX"));
	info.SetCopyright(wxT("(C) 2009,2010 using GPL v2 or later"));
	info.SetName(APPLICATION_NAME);
	info.SetVersion(VERSION);
	info.SetDescription(_("This program configures the Analogue Repeater software."));

	::wxAboutBox(info);
}
