/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "AnalogueRepeaterConfig.h"

static const wxString KEY_CALLSIGN            = wxT("/callsign");
static const wxString KEY_BEACON              = wxT("/beacon");
static const wxString KEY_ID_SPEED            = wxT("/idSpeed");
static const wxString KEY_ID_FREQ             = wxT("/idFreq");
static const wxString KEY_ID_LEVEL1           = wxT("/idLevel1");
static const wxString KEY_ID_LEVEL2           = wxT("/idLevel2");

static const wxString KEY_ACK_RADIO           = wxT("/ackRadio");
static const wxString KEY_ACK_EXTERNAL        = wxT("/ackExternal");
static const wxString KEY_ACK_SPEED           = wxT("/ackSpeed");
static const wxString KEY_ACK_FREQ            = wxT("/ackFreq");
static const wxString KEY_ACK_LEVEL           = wxT("/ackLevel");
static const wxString KEY_ACK_DELAY           = wxT("/ackDelay");
static const wxString KEY_ACK_MIN             = wxT("/ackMin");

static const wxString KEY_CALLSIGNTIME        = wxT("/callsignTime");
static const wxString KEY_TIMEOUT             = wxT("/timeout");
static const wxString KEY_LOCKOUTTIME         = wxT("/lockoutTime");
static const wxString KEY_HANGTIME            = wxT("/hangTime");
static const wxString KEY_LATCHTIME           = wxT("/latchTime");

static const wxString KEY_TONEBURST_ENABLE    = wxT("/toneburstEnable");
static const wxString KEY_TONEBURST_THRESHOLD = wxT("/toneburstThresh");
static const wxString KEY_CTCSS_FREQ          = wxT("/ctcssFreq");
static const wxString KEY_CTCSS_INTERNAL      = wxT("/ctcssInternal");
static const wxString KEY_CTCSS_THRESHOLD     = wxT("/ctcssThresh");
static const wxString KEY_CTCSS_LEVEL         = wxT("/ctcssLevel");
static const wxString KEY_CTCSS_OUTPUT        = wxT("/ctcssOutput");

static const wxString KEY_CALLSIGN_AT_START   = wxT("/callAtStart");
static const wxString KEY_CALLSIGN_AT_END     = wxT("/callAtEnd");
static const wxString KEY_TIMEOUT_TYPE        = wxT("/timeoutType");
static const wxString KEY_CALLSIGN_HOLDOFF    = wxT("/callHoldoff");

static const wxString KEY_RADIO_READ_DEVICE   = wxT("/radioReadDevice");
static const wxString KEY_RADIO_WRITE_DEVICE  = wxT("/radioWriteDevice");
static const wxString KEY_RADIO_AUDIO_DELAY   = wxT("/radioAudioDelay");
static const wxString KEY_RADIO_DEEMPHASIS    = wxT("/radioDeEmphasis");
static const wxString KEY_RADIO_PREEMPHASIS   = wxT("/radioPreEmphasis");
static const wxString KEY_RADIO_VOGAD         = wxT("/radioVOGAD");

static const wxString KEY_EXTERNAL_MODE             = wxT("/externalMode");
static const wxString KEY_EXTERNAL_READ_DEVICE      = wxT("/externalReadDevice");
static const wxString KEY_EXTERNAL_WRITE_DEVICE     = wxT("/externalWriteDevice");
static const wxString KEY_EXTERNAL_VOGAD            = wxT("/externalVOGAD");
static const wxString KEY_EXTERNAL_INTERFACE_DEVICE = wxT("/externalInterfaceDevice");
static const wxString KEY_EXTERNAL_INTERFACE_TX_PIN = wxT("/externalInterfaceTXPin");
static const wxString KEY_EXTERNAL_INTERFACE_RX_PIN = wxT("/externalInterfaceRXPin");

static const wxString KEY_IFACE_TYPE         = wxT("/interfaceType");
static const wxString KEY_PTT_DELAY          = wxT("/pttDelay");
static const wxString KEY_PTT_INVERT         = wxT("/pttInvert");
static const wxString KEY_SQUELCH_INVERT     = wxT("/squelchInvert");

static const wxString KEY_DTMF_RADIO     = wxT("/dtmfRadio");
static const wxString KEY_DTMF_EXTERNAL  = wxT("/dtmfExternal");
static const wxString KEY_DTMF_SHUTDOWN  = wxT("/dtmfShutdown");
static const wxString KEY_DTMF_STARTUP   = wxT("/dtmfStartup");
static const wxString KEY_DTMF_TIMEOUT   = wxT("/dtmfTimeout");
static const wxString KEY_DTMF_TIMERESET = wxT("/dtmfTimeReset");
static const wxString KEY_DTMF_COMMAND1      = wxT("/dtmfCommand1");
static const wxString KEY_DTMF_COMMAND1_LINE = wxT("/dtmfCommand1Line");
static const wxString KEY_DTMF_COMMAND2      = wxT("/dtmfCommand2");
static const wxString KEY_DTMF_COMMAND2_LINE = wxT("/dtmfCommand2Line");
static const wxString KEY_DTMF_OUTPUT1   = wxT("/dtmfOutput1");
static const wxString KEY_DTMF_OUTPUT2   = wxT("/dtmfOutput2");
static const wxString KEY_DTMF_OUTPUT3   = wxT("/dtmfOutput3");
static const wxString KEY_DTMF_OUTPUT4   = wxT("/dtmfOutput4");
static const wxString KEY_DTMF_THRESHOLD = wxT("/dtmfThreshold");

static const wxString KEY_ACTIVE_HANG_TIME = wxT("/activeHangTime");

static const wxString KEY_OUTPUT1 = wxT("/output1");
static const wxString KEY_OUTPUT2 = wxT("/output2");
static const wxString KEY_OUTPUT3 = wxT("/output3");
static const wxString KEY_OUTPUT4 = wxT("/output4");

static const wxString DEFAULT_CALLSIGN         = wxT("GB3IN B");
static const wxString DEFAULT_BEACON           = wxT("GB3IN B  IO93GD");
static const long     DEFAULT_ID_SPEED         = 25L;
static const long     DEFAULT_ID_FREQ          = 1000L;
static const double   DEFAULT_ID_LEVEL1        = 0.8;
static const double   DEFAULT_ID_LEVEL2        = 0.5;

static const wxString DEFAULT_ACK_RADIO        = wxT("K");
static const wxString DEFAULT_ACK_EXTERNAL     = wxT("N");
static const long     DEFAULT_ACK_SPEED        = 25L;
static const long     DEFAULT_ACK_FREQ         = 1750L;
static const double   DEFAULT_ACK_LEVEL        = 0.8;
static const long     DEFAULT_ACK_DELAY        = 1L;
static const long     DEFAULT_ACK_MIN          = 5L;

static const long     DEFAULT_CALLSIGNTIME     = 600L;
static const long     DEFAULT_TIMEOUT          = 120L;
static const long     DEFAULT_LOCKOUTTIME      = 30L;
static const long     DEFAULT_HANGTIME         = 7L;
static const long     DEFAULT_LATCHTIME        = 3L;

static const bool                  DEFAULT_TONEBURST_ENABLE = true;
static const double                DEFAULT_TONEBURST_THRESH = 0.5;
static const double                DEFAULT_CTCSS_FREQ       = 71.9;
static const bool                  DEFAULT_CTCSS_INTERNAL   = true;
static const double                DEFAULT_CTCSS_THRESH     = 0.5;
static const double                DEFAULT_CTCSS_LEVEL      = 0.5;
static const ANALOGUE_CTCSS_OUTPUT DEFAULT_CTCSS_OUTPUT     = ACO_WHEN_OPEN;

static const ANALOGUE_CALLSIGN_START   DEFAULT_CALLSIGN_AT_START = ACS_NONE;
static const bool                      DEFAULT_CALLSIGN_AT_END   = true;
static const ANALOGUE_TIMEOUT_TYPE     DEFAULT_TIMEOUT_TYPE      = ATT_DL;
static const ANALOGUE_CALLSIGN_HOLDOFF DEFAULT_CALLSIGN_HOLDOFF  = ACH_NONE;

static const wxString DEFAULT_RADIO_READ_DEVICE    = wxEmptyString;
static const wxString DEFAULT_RADIO_WRITE_DEVICE   = wxEmptyString;
static const long     DEFAULT_RADIO_AUDIO_DELAY    = 10L;
static const bool     DEFAULT_RADIO_DEEMPHASIS     = false;
static const bool     DEFAULT_RADIO_PREEMPHASIS    = false;
static const bool     DEFAULT_RADIO_VOGAD          = false;

static const ANALOGUE_EXTERNAL_MODE DEFAULT_EXTERNAL_MODE             = AEM_DISABLED;
static const wxString               DEFAULT_EXTERNAL_READ_DEVICE      = wxEmptyString;
static const wxString               DEFAULT_EXTERNAL_WRITE_DEVICE     = wxEmptyString;
static const bool                   DEFAULT_EXTERNAL_VOGAD            = false;
static const wxString               DEFAULT_EXTERNAL_INTERFACE_DEVICE = wxEmptyString;
static const SERIALPIN              DEFAULT_EXTERNAL_INTERFACE_TX_PIN = SERIAL_DTR;
static const SERIALPIN              DEFAULT_EXTERNAL_INTERFACE_RX_PIN = SERIAL_CD;

static const wxString DEFAULT_IFACE_TYPE     = wxEmptyString;
static const long     DEFAULT_PTT_DELAY      = 5L;
static const bool     DEFAULT_PTT_INVERT     = false;
static const bool     DEFAULT_SQUELCH_INVERT = false;

static const bool     DEFAULT_DTMF_RADIO     = false;
static const bool     DEFAULT_DTMF_EXTERNAL  = false;
static const wxString DEFAULT_DTMF_SHUTDOWN  = wxEmptyString;
static const wxString DEFAULT_DTMF_STARTUP   = wxEmptyString;
static const wxString DEFAULT_DTMF_TIMEOUT   = wxEmptyString;
static const wxString DEFAULT_DTMF_TIMERESET = wxEmptyString;
static const wxString DEFAULT_DTMF_COMMAND1      = wxEmptyString;
static const wxString DEFAULT_DTMF_COMMAND1_LINE = wxEmptyString;
static const wxString DEFAULT_DTMF_COMMAND2      = wxEmptyString;
static const wxString DEFAULT_DTMF_COMMAND2_LINE = wxEmptyString;
static const wxString DEFAULT_DTMF_OUTPUT1   = wxEmptyString;
static const wxString DEFAULT_DTMF_OUTPUT2   = wxEmptyString;
static const wxString DEFAULT_DTMF_OUTPUT3   = wxEmptyString;
static const wxString DEFAULT_DTMF_OUTPUT4   = wxEmptyString;
static const double   DEFAULT_DTMF_THRESHOLD = 0.5;

static const bool     DEFAULT_OUTPUT1 = false;
static const bool     DEFAULT_OUTPUT2 = false;
static const bool     DEFAULT_OUTPUT3 = false;
static const bool     DEFAULT_OUTPUT4 = false;

static const long     DEFAULT_ACTIVE_HANG_TIME = 0L;


CAnalogueRepeaterConfig::CAnalogueRepeaterConfig(wxConfigBase* config, const wxString& name) :
m_config(config),
m_name()
{
	wxASSERT(config != NULL);

	if (!name.IsEmpty())
		m_name = wxT("/") + name;
}

CAnalogueRepeaterConfig::~CAnalogueRepeaterConfig()
{
	delete m_config;
}

void CAnalogueRepeaterConfig::getCallsign(wxString& callsign, wxString& beacon, unsigned int& speed, unsigned int& freq, wxFloat32& level1, wxFloat32& level2) const
{
	wxString callsignKey = m_name + KEY_CALLSIGN;
	wxString beaconKey   = m_name + KEY_BEACON;
	wxString idSpeedKey  = m_name + KEY_ID_SPEED;
	wxString idFreqKey   = m_name + KEY_ID_FREQ;
	wxString idLevel1Key = m_name + KEY_ID_LEVEL1;
	wxString idLevel2Key = m_name + KEY_ID_LEVEL2;

	m_config->Read(callsignKey, &callsign, DEFAULT_CALLSIGN);
	m_config->Read(beaconKey,   &beacon,   DEFAULT_BEACON);

	long temp1;
	m_config->Read(idSpeedKey, &temp1, DEFAULT_ID_SPEED);
	speed = (unsigned int)temp1;

	m_config->Read(idFreqKey, &temp1, DEFAULT_ID_FREQ);
	freq = (unsigned int)temp1;

	double temp2;
	m_config->Read(idLevel1Key, &temp2, DEFAULT_ID_LEVEL1);
	level1 = wxFloat32(temp2);

	m_config->Read(idLevel2Key, &temp2, DEFAULT_ID_LEVEL2);
	level2 = wxFloat32(temp2);
}

void CAnalogueRepeaterConfig::setCallsign(const wxString& callsign, const wxString& beacon, unsigned int speed, unsigned int freq, wxFloat32 level1, wxFloat32 level2) const
{
	wxString callsignKey = m_name + KEY_CALLSIGN;
	wxString beaconKey   = m_name + KEY_BEACON;
	wxString idSpeedKey  = m_name + KEY_ID_SPEED;
	wxString idFreqKey   = m_name + KEY_ID_FREQ;
	wxString idLevel1Key = m_name + KEY_ID_LEVEL1;
	wxString idLevel2Key = m_name + KEY_ID_LEVEL2;

	m_config->Write(callsignKey,   callsign);
	m_config->Write(beaconKey,   beacon);
	m_config->Write(idSpeedKey,  long(speed));
	m_config->Write(idFreqKey,   long(freq));
	m_config->Write(idLevel1Key, double(level1));
	m_config->Write(idLevel2Key, double(level2));
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getAck(wxString& radioAck, wxString& extAck, unsigned int& speed, unsigned int& freq, wxFloat32& level, unsigned int& ack, unsigned int& minimum) const
{
	wxString ackRadioKey  = m_name + KEY_ACK_RADIO;
	wxString ackExtKey    = m_name + KEY_ACK_EXTERNAL;
	wxString ackSpeedKey  = m_name + KEY_ACK_SPEED;
	wxString ackFreqKey   = m_name + KEY_ACK_FREQ;
	wxString ackLevelKey  = m_name + KEY_ACK_LEVEL;
	wxString ackDelayKey  = m_name + KEY_ACK_DELAY;
	wxString ackMinKey    = m_name + KEY_ACK_MIN;

	m_config->Read(ackRadioKey, &radioAck, DEFAULT_ACK_RADIO);
	m_config->Read(ackExtKey,   &extAck,   DEFAULT_ACK_EXTERNAL);

	long temp1;
	m_config->Read(ackSpeedKey, &temp1, DEFAULT_ACK_SPEED);
	speed = (unsigned int)temp1;

	m_config->Read(ackFreqKey, &temp1, DEFAULT_ACK_FREQ);
	freq = (unsigned int)temp1;

	double temp2;
	m_config->Read(ackLevelKey, &temp2, DEFAULT_ACK_LEVEL);
	level = wxFloat32(temp2);

	m_config->Read(ackDelayKey, &temp1, DEFAULT_ACK_DELAY);
	ack = (unsigned int)temp1;

	m_config->Read(ackMinKey, &temp1, DEFAULT_ACK_MIN);
	minimum = (unsigned int)temp1;
}

void CAnalogueRepeaterConfig::setAck(const wxString& radioAck, const wxString& extAck, unsigned int speed, unsigned int freq, wxFloat32 level, unsigned int ack, unsigned int minimum) const
{
	wxString ackRadioKey  = m_name + KEY_ACK_RADIO;
	wxString ackExtKey    = m_name + KEY_ACK_EXTERNAL;
	wxString ackSpeedKey  = m_name + KEY_ACK_SPEED;
	wxString ackFreqKey   = m_name + KEY_ACK_FREQ;
	wxString ackLevelKey  = m_name + KEY_ACK_LEVEL;
	wxString ackDelayKey  = m_name + KEY_ACK_DELAY;
	wxString ackMinKey    = m_name + KEY_ACK_MIN;

	m_config->Write(ackRadioKey, radioAck);
	m_config->Write(ackExtKey,   extAck);
	m_config->Write(ackSpeedKey, long(speed));
	m_config->Write(ackFreqKey,  long(freq));
	m_config->Write(ackLevelKey, double(level));
	m_config->Write(ackDelayKey, long(ack));
	m_config->Write(ackMinKey,   long(minimum));
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getTimes(unsigned int& callsignTime, unsigned int& timeout, unsigned int& lockoutTime, unsigned int& hangTime, unsigned int& latchTime) const
{
	wxString callsignTimeKey = m_name + KEY_CALLSIGNTIME;
	wxString timeoutKey      = m_name + KEY_TIMEOUT;
	wxString lockoutTimeKey  = m_name + KEY_LOCKOUTTIME;
	wxString hangTimeKey     = m_name + KEY_HANGTIME;
	wxString latchTimeKey    = m_name + KEY_LATCHTIME;

	long temp;
	m_config->Read(callsignTimeKey, &temp, DEFAULT_CALLSIGNTIME);
	callsignTime = (unsigned int)temp;

	m_config->Read(timeoutKey, &temp, DEFAULT_TIMEOUT);
	timeout = (unsigned int)temp;

	m_config->Read(lockoutTimeKey, &temp, DEFAULT_LOCKOUTTIME);
	lockoutTime = (unsigned int)temp;

	m_config->Read(hangTimeKey, &temp, DEFAULT_HANGTIME);
	hangTime = (unsigned int)temp;

	m_config->Read(latchTimeKey, &temp, DEFAULT_LATCHTIME);
	latchTime = (unsigned int)temp;
}

void CAnalogueRepeaterConfig::setTimes(unsigned int callsignTime, unsigned int timeout, unsigned int lockoutTime, unsigned int hangTime, unsigned int latchTime) const
{
	wxString callsignTimeKey = m_name + KEY_CALLSIGNTIME;
	wxString timeoutKey      = m_name + KEY_TIMEOUT;
	wxString lockoutTimeKey  = m_name + KEY_LOCKOUTTIME;
	wxString hangTimeKey     = m_name + KEY_HANGTIME;
	wxString latchTimeKey    = m_name + KEY_LATCHTIME;

	m_config->Write(callsignTimeKey, long(callsignTime));
	m_config->Write(timeoutKey,      long(timeout));
	m_config->Write(lockoutTimeKey,  long(lockoutTime));
	m_config->Write(hangTimeKey,     long(hangTime));
	m_config->Write(latchTimeKey,    long(latchTime));
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getTones(bool& tbEnable, wxFloat32& tbThreshold, wxFloat32& ctcssFreq, bool& ctcssInternal, wxFloat32& ctcssThresh, wxFloat32& ctcssLevel, ANALOGUE_CTCSS_OUTPUT& ctcssOutput) const
{
	wxString tbEnableKey       = m_name + KEY_TONEBURST_ENABLE;
	wxString tbThresholdKey    = m_name + KEY_TONEBURST_THRESHOLD;
	wxString ctcssFreqKey      = m_name + KEY_CTCSS_FREQ;
	wxString ctcssInternalKey  = m_name + KEY_CTCSS_INTERNAL;
	wxString ctcssThresholdKey = m_name + KEY_CTCSS_THRESHOLD;
	wxString ctcssLevelKey     = m_name + KEY_CTCSS_LEVEL;
	wxString ctcssOutputKey    = m_name + KEY_CTCSS_OUTPUT;

	m_config->Read(tbEnableKey, &tbEnable, DEFAULT_TONEBURST_ENABLE);

	double temp1;
	m_config->Read(tbThresholdKey, &temp1, DEFAULT_TONEBURST_THRESH);
	tbThreshold = wxFloat32(temp1);

	m_config->Read(ctcssFreqKey, &temp1, DEFAULT_CTCSS_FREQ);
	ctcssFreq = wxFloat32(temp1);

	m_config->Read(ctcssInternalKey, &ctcssInternal, DEFAULT_CTCSS_INTERNAL);

	m_config->Read(ctcssThresholdKey, &temp1, DEFAULT_CTCSS_THRESH);
	ctcssThresh = wxFloat32(temp1);

	m_config->Read(ctcssLevelKey, &temp1, DEFAULT_CTCSS_LEVEL);
	ctcssLevel = wxFloat32(temp1);

	long temp2;
	m_config->Read(ctcssOutputKey, &temp2, long(DEFAULT_CTCSS_OUTPUT));
	ctcssOutput = ANALOGUE_CTCSS_OUTPUT(temp2);
}

void CAnalogueRepeaterConfig::setTones(bool tbEnable, wxFloat32 tbThreshold, wxFloat32 ctcssFreq, bool ctcssInternal, wxFloat32 ctcssThresh, wxFloat32 ctcssLevel, ANALOGUE_CTCSS_OUTPUT ctcssOutput) const
{
	wxString tbEnableKey       = m_name + KEY_TONEBURST_ENABLE;
	wxString tbThreshKey       = m_name + KEY_TONEBURST_THRESHOLD;
	wxString ctcssFreqKey      = m_name + KEY_CTCSS_FREQ;
	wxString ctcssInternalKey  = m_name + KEY_CTCSS_INTERNAL;
	wxString ctcssThresholdKey = m_name + KEY_CTCSS_THRESHOLD;
	wxString ctcssLevelKey     = m_name + KEY_CTCSS_LEVEL;
	wxString ctcssOutputKey    = m_name + KEY_CTCSS_OUTPUT;

	m_config->Write(tbEnableKey,       tbEnable);
	m_config->Write(tbThreshKey,       double(tbThreshold));
	m_config->Write(ctcssFreqKey,      double(ctcssFreq));
	m_config->Write(ctcssInternalKey,  ctcssInternal);
	m_config->Write(ctcssThresholdKey, double(ctcssThresh));
	m_config->Write(ctcssLevelKey,     double(ctcssLevel));
	m_config->Write(ctcssOutputKey,    long(ctcssOutput));
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getFeel(ANALOGUE_CALLSIGN_START& callsignAtStart, bool& callsignAtEnd, ANALOGUE_TIMEOUT_TYPE& timeoutType, ANALOGUE_CALLSIGN_HOLDOFF& callsignHoldoff) const
{
	wxString callsignAtStartKey = m_name + KEY_CALLSIGN_AT_START;
	wxString callsignAtEndKey   = m_name + KEY_CALLSIGN_AT_END;
	wxString timeoutTypeKey     = m_name + KEY_TIMEOUT_TYPE;
	wxString callsignHoldoffKey = m_name + KEY_CALLSIGN_HOLDOFF;

	long temp;
	m_config->Read(callsignAtStartKey, &temp,     long(DEFAULT_CALLSIGN_AT_START));
	callsignAtStart = ANALOGUE_CALLSIGN_START(temp);

	m_config->Read(callsignAtEndKey,   &callsignAtEnd, DEFAULT_CALLSIGN_AT_END);

	m_config->Read(timeoutTypeKey,     &temp,     long(DEFAULT_TIMEOUT_TYPE));
	timeoutType = ANALOGUE_TIMEOUT_TYPE(temp);

	m_config->Read(callsignHoldoffKey, &temp,     long(DEFAULT_CALLSIGN_HOLDOFF));
	callsignHoldoff = ANALOGUE_CALLSIGN_HOLDOFF(temp);
}

void CAnalogueRepeaterConfig::setFeel(ANALOGUE_CALLSIGN_START callsignAtStart, bool callsignAtEnd, ANALOGUE_TIMEOUT_TYPE timeoutType, ANALOGUE_CALLSIGN_HOLDOFF callsignHoldoff) const
{
	wxString callsignAtStartKey = m_name + KEY_CALLSIGN_AT_START;
	wxString callsignAtEndKey   = m_name + KEY_CALLSIGN_AT_END;
	wxString timeoutTypeKey     = m_name + KEY_TIMEOUT_TYPE;
	wxString callsignHoldoffKey = m_name + KEY_CALLSIGN_HOLDOFF;

	m_config->Write(callsignAtStartKey, long(callsignAtStart));
	m_config->Write(callsignAtEndKey,   callsignAtEnd);
	m_config->Write(timeoutTypeKey,     long(timeoutType));
	m_config->Write(callsignHoldoffKey, long(callsignHoldoff));
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getRadio(wxString& readDevice, wxString& writeDevice, unsigned int& delay, bool& deEmphasis, bool& preEmphasis, bool& vogad) const
{
	wxString readDeviceKey  = m_name + KEY_RADIO_READ_DEVICE;
	wxString writeDeviceKey = m_name + KEY_RADIO_WRITE_DEVICE;
	wxString audioDelayKey  = m_name + KEY_RADIO_AUDIO_DELAY;
	wxString deEmphasisKey  = m_name + KEY_RADIO_DEEMPHASIS;
	wxString preEmphasisKey = m_name + KEY_RADIO_PREEMPHASIS;
	wxString vogadKey       = m_name + KEY_RADIO_VOGAD;

	m_config->Read(readDeviceKey,  &readDevice,  DEFAULT_RADIO_READ_DEVICE);
	m_config->Read(writeDeviceKey, &writeDevice, DEFAULT_RADIO_WRITE_DEVICE);

	long temp;
	m_config->Read(audioDelayKey,  &temp,        DEFAULT_RADIO_AUDIO_DELAY);
	delay = (unsigned int)temp;

	m_config->Read(deEmphasisKey,  &deEmphasis,  DEFAULT_RADIO_DEEMPHASIS);
	m_config->Read(preEmphasisKey, &preEmphasis, DEFAULT_RADIO_PREEMPHASIS);

	m_config->Read(vogadKey, &vogad, DEFAULT_RADIO_VOGAD);
}

void CAnalogueRepeaterConfig::setRadio(const wxString& readDevice, const wxString& writeDevice, unsigned int delay, bool deEmphasis, bool preEmphasis, bool vogad) const
{
	wxString readDeviceKey  = m_name + KEY_RADIO_READ_DEVICE;
	wxString writeDeviceKey = m_name + KEY_RADIO_WRITE_DEVICE;
	wxString audioDelayKey  = m_name + KEY_RADIO_AUDIO_DELAY;
	wxString deEmphasisKey  = m_name + KEY_RADIO_DEEMPHASIS;
	wxString preEmphasisKey = m_name + KEY_RADIO_PREEMPHASIS;
	wxString vogadKey       = m_name + KEY_RADIO_VOGAD;

	m_config->Write(readDeviceKey,  readDevice);
	m_config->Write(writeDeviceKey, writeDevice);
	m_config->Write(audioDelayKey,  long(delay));
	m_config->Write(deEmphasisKey,  deEmphasis);
	m_config->Write(preEmphasisKey, preEmphasis);
	m_config->Write(vogadKey,       vogad);
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getExternal(ANALOGUE_EXTERNAL_MODE& mode, wxString& readDevice, wxString& writeDevice, bool& vogad, wxString& device, SERIALPIN& txPin, SERIALPIN& rxPin) const
{
	wxString modeKey        = m_name + KEY_EXTERNAL_MODE;
	wxString readDeviceKey  = m_name + KEY_EXTERNAL_READ_DEVICE;
	wxString writeDeviceKey = m_name + KEY_EXTERNAL_WRITE_DEVICE;
	wxString vogadKey       = m_name + KEY_EXTERNAL_VOGAD;
	wxString deviceKey      = m_name + KEY_EXTERNAL_INTERFACE_DEVICE;
	wxString txPinKey       = m_name + KEY_EXTERNAL_INTERFACE_TX_PIN;
	wxString rxPinKey       = m_name + KEY_EXTERNAL_INTERFACE_RX_PIN;

	long temp;
	m_config->Read(modeKey,        &temp,        long(DEFAULT_EXTERNAL_MODE));
	mode = ANALOGUE_EXTERNAL_MODE(temp);

	m_config->Read(readDeviceKey,  &readDevice,  DEFAULT_EXTERNAL_READ_DEVICE);
	m_config->Read(writeDeviceKey, &writeDevice, DEFAULT_EXTERNAL_WRITE_DEVICE);

	m_config->Read(vogadKey,       &vogad,       DEFAULT_EXTERNAL_VOGAD);

	m_config->Read(deviceKey,      &device,      DEFAULT_EXTERNAL_INTERFACE_DEVICE);

	m_config->Read(txPinKey,       &temp,        long(DEFAULT_EXTERNAL_INTERFACE_TX_PIN));
	txPin = SERIALPIN(temp);

	m_config->Read(rxPinKey,       &temp,        long(DEFAULT_EXTERNAL_INTERFACE_RX_PIN));
	rxPin = SERIALPIN(temp);
}

void CAnalogueRepeaterConfig::setExternal(ANALOGUE_EXTERNAL_MODE mode, const wxString& readDevice, const wxString& writeDevice, bool vogad, const wxString& device, SERIALPIN txPin, SERIALPIN rxPin) const
{
	wxString modeKey        = m_name + KEY_EXTERNAL_MODE;
	wxString readDeviceKey  = m_name + KEY_EXTERNAL_READ_DEVICE;
	wxString writeDeviceKey = m_name + KEY_EXTERNAL_WRITE_DEVICE;
	wxString vogadKey       = m_name + KEY_EXTERNAL_VOGAD;
	wxString deviceKey      = m_name + KEY_EXTERNAL_INTERFACE_DEVICE;
	wxString txPinKey       = m_name + KEY_EXTERNAL_INTERFACE_TX_PIN;
	wxString rxPinKey       = m_name + KEY_EXTERNAL_INTERFACE_RX_PIN;

	m_config->Write(modeKey,        long(mode));
	m_config->Write(readDeviceKey,  readDevice);
	m_config->Write(writeDeviceKey, writeDevice);
	m_config->Write(vogadKey,       vogad);
	m_config->Write(deviceKey,      device);
	m_config->Write(txPinKey,       long(txPin));
	m_config->Write(rxPinKey,       long(rxPin));
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getController(wxString& type, unsigned int& pttDelay, bool& pttInvert, bool& squelchInvert) const
{
	wxString ifaceTypeKey     = m_name + KEY_IFACE_TYPE;
	wxString pttDelayKey      = m_name + KEY_PTT_DELAY;
	wxString pttInvertKey     = m_name + KEY_PTT_INVERT;
	wxString squelchInvertKey = m_name + KEY_SQUELCH_INVERT;

	m_config->Read(ifaceTypeKey,     &type,          DEFAULT_IFACE_TYPE);

	long temp;
	m_config->Read(pttDelayKey,      &temp,          DEFAULT_PTT_DELAY);
	pttDelay = (unsigned int)temp;

	m_config->Read(pttInvertKey,     &pttInvert,     DEFAULT_PTT_INVERT);
	m_config->Read(squelchInvertKey, &squelchInvert, DEFAULT_SQUELCH_INVERT);
}

void CAnalogueRepeaterConfig::setController(const wxString& type, unsigned int pttDelay, bool pttInvert, bool squelchInvert) const
{
	wxString ifaceTypeKey     = m_name + KEY_IFACE_TYPE;
	wxString pttDelayKey      = m_name + KEY_PTT_DELAY;
	wxString pttInvertKey     = m_name + KEY_PTT_INVERT;
	wxString squelchInvertKey = m_name + KEY_SQUELCH_INVERT;

	m_config->Write(ifaceTypeKey,     type);
	m_config->Write(pttDelayKey,      long(pttDelay));
	m_config->Write(pttInvertKey,     pttInvert);
	m_config->Write(squelchInvertKey, squelchInvert);
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getDTMF(bool& radio, bool& external, wxString& shutdown, wxString& startup, wxString& timeout, wxString& timeReset, wxString& command1, wxString& command1Line, wxString& command2, wxString& command2Line, wxString& output1, wxString& output2, wxString& output3, wxString& output4, wxFloat32& threshold) const
{
	wxString radioKey        = m_name + KEY_DTMF_RADIO;
	wxString externalKey     = m_name + KEY_DTMF_EXTERNAL;
	wxString shutdownKey     = m_name + KEY_DTMF_SHUTDOWN;
	wxString startupKey      = m_name + KEY_DTMF_STARTUP;
	wxString timeoutKey      = m_name + KEY_DTMF_TIMEOUT;
	wxString timeResetKey    = m_name + KEY_DTMF_TIMERESET;
	wxString command1Key     = m_name + KEY_DTMF_COMMAND1;
	wxString command1LineKey = m_name + KEY_DTMF_COMMAND1_LINE;
	wxString command2Key     = m_name + KEY_DTMF_COMMAND2;
	wxString command2LineKey = m_name + KEY_DTMF_COMMAND2_LINE;
	wxString output1Key      = m_name + KEY_DTMF_OUTPUT1;
	wxString output2Key      = m_name + KEY_DTMF_OUTPUT2;
	wxString output3Key      = m_name + KEY_DTMF_OUTPUT3;
	wxString output4Key      = m_name + KEY_DTMF_OUTPUT4;
	wxString thresholdKey    = m_name + KEY_DTMF_THRESHOLD;

	m_config->Read(radioKey,        &radio,        DEFAULT_DTMF_RADIO);
	m_config->Read(externalKey,     &external,     DEFAULT_DTMF_EXTERNAL);
	m_config->Read(shutdownKey,     &shutdown,     DEFAULT_DTMF_SHUTDOWN);
	m_config->Read(startupKey,      &startup,      DEFAULT_DTMF_STARTUP);
	m_config->Read(timeoutKey,      &timeout,      DEFAULT_DTMF_TIMEOUT);
	m_config->Read(timeResetKey,    &timeReset,    DEFAULT_DTMF_TIMERESET);
	m_config->Read(command1Key,     &command1,     DEFAULT_DTMF_COMMAND1);
	m_config->Read(command1LineKey, &command1Line, DEFAULT_DTMF_COMMAND1_LINE);
	m_config->Read(command2Key,     &command2,     DEFAULT_DTMF_COMMAND2);
	m_config->Read(command2LineKey, &command2Line, DEFAULT_DTMF_COMMAND2_LINE);
	m_config->Read(output1Key,      &output1,      DEFAULT_DTMF_OUTPUT1);
	m_config->Read(output2Key,      &output2,      DEFAULT_DTMF_OUTPUT2);
	m_config->Read(output3Key,      &output3,      DEFAULT_DTMF_OUTPUT3);
	m_config->Read(output4Key,      &output4,      DEFAULT_DTMF_OUTPUT4);

	double temp;
	m_config->Read(thresholdKey, &temp,      DEFAULT_DTMF_THRESHOLD);
	threshold = wxFloat32(temp);
}

void CAnalogueRepeaterConfig::setDTMF(bool radio, bool external, const wxString& shutdown, const wxString& startup, const wxString& timeout, const wxString& timeReset, const wxString& command1, const wxString& command1Line, const wxString& command2, const wxString& command2Line, const wxString& output1, const wxString& output2, const wxString& output3, const wxString& output4, wxFloat32 threshold) const
{
	wxString radioKey        = m_name + KEY_DTMF_RADIO;
	wxString externalKey     = m_name + KEY_DTMF_EXTERNAL;
	wxString shutdownKey     = m_name + KEY_DTMF_SHUTDOWN;
	wxString startupKey      = m_name + KEY_DTMF_STARTUP;
	wxString timeoutKey      = m_name + KEY_DTMF_TIMEOUT;
	wxString timeResetKey    = m_name + KEY_DTMF_TIMERESET;
	wxString command1Key     = m_name + KEY_DTMF_COMMAND1;
	wxString command1LineKey = m_name + KEY_DTMF_COMMAND1_LINE;
	wxString command2Key     = m_name + KEY_DTMF_COMMAND2;
	wxString command2LineKey = m_name + KEY_DTMF_COMMAND2_LINE;
	wxString output1Key      = m_name + KEY_DTMF_OUTPUT1;
	wxString output2Key      = m_name + KEY_DTMF_OUTPUT2;
	wxString output3Key      = m_name + KEY_DTMF_OUTPUT3;
	wxString output4Key      = m_name + KEY_DTMF_OUTPUT4;
	wxString thresholdKey    = m_name + KEY_DTMF_THRESHOLD;

	m_config->Write(radioKey,        radio);
	m_config->Write(externalKey,     external);
	m_config->Write(shutdownKey,     shutdown);
	m_config->Write(startupKey,      startup);
	m_config->Write(timeoutKey,      timeout);
	m_config->Write(timeResetKey,    timeReset);
	m_config->Write(command1Key,     command1);
	m_config->Write(command1LineKey, command1Line);
	m_config->Write(command2Key,     command2);
	m_config->Write(command2LineKey, command2Line);
	m_config->Write(output1Key,      output1);
	m_config->Write(output2Key,      output2);
	m_config->Write(output3Key,      output3);
	m_config->Write(output4Key,      output4);
	m_config->Write(thresholdKey,    double(threshold));
	m_config->Flush();
}

void CAnalogueRepeaterConfig::getOutputs(bool& out1, bool& out2, bool& out3, bool& out4) const
{
	wxString output1Key = m_name + KEY_OUTPUT1;
	wxString output2Key = m_name + KEY_OUTPUT2;
	wxString output3Key = m_name + KEY_OUTPUT3;
	wxString output4Key = m_name + KEY_OUTPUT4;

	m_config->Read(output1Key, &out1, DEFAULT_OUTPUT1);
	m_config->Read(output2Key, &out2, DEFAULT_OUTPUT2);
	m_config->Read(output3Key, &out3, DEFAULT_OUTPUT3);
	m_config->Read(output4Key, &out4, DEFAULT_OUTPUT4);
}

void CAnalogueRepeaterConfig::setOutputs(bool out1, bool out2, bool out3, bool out4) const
{
	wxString output1Key = m_name + KEY_OUTPUT1;
	wxString output2Key = m_name + KEY_OUTPUT2;
	wxString output3Key = m_name + KEY_OUTPUT3;
	wxString output4Key = m_name + KEY_OUTPUT4;

	m_config->Write(output1Key, out1);
	m_config->Write(output2Key, out2);
	m_config->Write(output3Key, out3);
	m_config->Write(output4Key, out4);
	m_config->Flush();
}


void CAnalogueRepeaterConfig::getActiveHang(unsigned int& time) const
{
	wxString timeKey = m_name + KEY_ACTIVE_HANG_TIME;

	long temp;
	m_config->Read(timeKey, &temp, DEFAULT_ACTIVE_HANG_TIME);
	time = (unsigned int)temp;
}

void CAnalogueRepeaterConfig::setActiveHang(unsigned int time) const
{
	wxString timeKey = m_name + KEY_ACTIVE_HANG_TIME;

	m_config->Write(timeKey, long(time));
	m_config->Flush();
}
