/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	AnalogueRepeaterConfig_H
#define	AnalogueRepeaterConfig_H

#include "AnalogueDefines.h"
#include "TimeoutTones.h"
#include "SerialController.h"

#include <wx/wx.h>
#include <wx/config.h>

class CAnalogueRepeaterConfig {

public:
	CAnalogueRepeaterConfig(wxConfigBase* config, const wxString& name);
	~CAnalogueRepeaterConfig();

	void getCallsign(wxString& callsign, wxString& beacon, unsigned int& speed, unsigned int& freq, wxFloat32& level1, wxFloat32& level2) const;
	void setCallsign(const wxString& callsign, const wxString& beacon, unsigned int speed, unsigned int freq, wxFloat32 level1, wxFloat32 level2) const;

	void getAck(wxString& radioAck, wxString& extAck, unsigned int& speed, unsigned int& freq, wxFloat32& level, unsigned int& ack, unsigned int& minimum) const;
	void setAck(const wxString& radioAck, const wxString& extAck, unsigned int speed, unsigned int freq, wxFloat32 level, unsigned int ack, unsigned int minimum) const;

	void getTimes(unsigned int& callsignTime, unsigned int& timeout, unsigned int& lockoutTime, unsigned int& hangTime, unsigned int& latchTime) const;
	void setTimes(unsigned int callsignTime, unsigned int timeout, unsigned int lockoutTime, unsigned int hangTime, unsigned int latchTime) const;

	void getTones(bool& tbEnable, wxFloat32& tbThreshold, wxFloat32& ctcssFreq, bool& ctcssInternal, wxFloat32& ctcssThresh, wxFloat32& ctcssLevel, ANALOGUE_CTCSS_OUTPUT& ctcssOutput) const;
	void setTones(bool tbEnable, wxFloat32 tbThreshold, wxFloat32 ctcssFreq, bool ctcssInternal, wxFloat32 ctcssThresh, wxFloat32 ctcssLevel, ANALOGUE_CTCSS_OUTPUT ctcssOutput) const;

	void getFeel(ANALOGUE_CALLSIGN_START& callsignAtStart, bool& callsignAtEnd, ANALOGUE_TIMEOUT_TYPE& timeoutType, ANALOGUE_CALLSIGN_HOLDOFF& callsignHoldoff) const;
	void setFeel(ANALOGUE_CALLSIGN_START callsignAtStart, bool callsignAtEnd, ANALOGUE_TIMEOUT_TYPE timeoutType, ANALOGUE_CALLSIGN_HOLDOFF callsignHoldoff) const;

	void getRadio(wxString& readDevice, wxString& writeDevice, unsigned int& delay, bool& deEmphasis, bool& preEmphasis, bool& vogad) const;
	void setRadio(const wxString& readDevice, const wxString& writeDevice, unsigned int delay, bool deEmphasis, bool preEmphasis, bool vogad) const;

	void getExternal(ANALOGUE_EXTERNAL_MODE& mode, wxString& readDevice, wxString& writeDevice, bool& vogad, wxString& device, SERIALPIN& txPin, SERIALPIN& rxPin) const;
	void setExternal(ANALOGUE_EXTERNAL_MODE mode, const wxString& readDevice, const wxString& writeDevice, bool vogad, const wxString& device, SERIALPIN txPin, SERIALPIN rxPin) const;

	void getController(wxString& type, unsigned int& pttDelay, bool& pttInvert, bool& squelchInvert) const;
	void setController(const wxString& type, unsigned int pttDelay, bool pttInvert, bool squelchInvert) const;

	void getDTMF(bool& radio, bool& external, wxString& shutdown, wxString& startup, wxString& timeout,
		wxString& timeReset, wxString& command1, wxString& command1Line, wxString& command2, wxString& command2Line,
		wxString& output1, wxString& output2, wxString& output3, wxString& output4, wxFloat32& threshold) const;
	void setDTMF(bool radio, bool external, const wxString& shutdown, const wxString& startup, const wxString& timeout,
		const wxString& timeReset, const wxString& command1, const wxString& command1Line, const wxString& command2,
		const wxString& command2Line, const wxString& output1, const wxString& output2, const wxString& output3,
		const wxString& output4, wxFloat32 threshold) const;

	void getActiveHang(unsigned int& time) const;
	void setActiveHang(unsigned int time) const;

	void getOutputs(bool& out1, bool& out2, bool& out3, bool& out4) const;
	void setOutputs(bool out1, bool out2, bool out3, bool out4) const;

private:
	wxConfigBase* m_config;
	wxString      m_name;
};

#endif
