/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "DStarRepeaterConfig.h"

static const wxString  KEY_CALLSIGN           = wxT("/callsign");
static const wxString  KEY_GATEWAY            = wxT("/gateway");
static const wxString  KEY_MODE               = wxT("/mode");
static const wxString  KEY_ACK                = wxT("/ack");
static const wxString  KEY_GATEWAY_ADDRESS    = wxT("/gatewayAddress");
static const wxString  KEY_GATEWAY_PORT       = wxT("/gatewayPort");
static const wxString  KEY_LOCAL_ADDRESS      = wxT("/localAddress");
static const wxString  KEY_LOCAL_PORT         = wxT("/localPort");
static const wxString  KEY_NETWORK_DELAY      = wxT("/networkDelay");
static const wxString  KEY_SOUND_READ_DEVICE  = wxT("/readDevice");
static const wxString  KEY_SOUND_WRITE_DEVICE = wxT("/writeDevice");
static const wxString  KEY_LEVEL              = wxT("/level");
static const wxString  KEY_PTT_DELAY          = wxT("/pttDelay");
static const wxString  KEY_PTT_INVERT         = wxT("/pttInvert");
static const wxString  KEY_GMSK_TXINV         = wxT("/txInvert");
static const wxString  KEY_GMSK_RXINV         = wxT("/rxInvert");
static const wxString  KEY_IFACE_TYPE         = wxT("/interfaceType");
static const wxString  KEY_TIMEOUT            = wxT("/timeout");
static const wxString  KEY_ACK_TIME           = wxT("/ackTime");
static const wxString  KEY_CONTROL_ENABLED    = wxT("/controlEnabled");
static const wxString  KEY_CONTROL_RPT1       = wxT("/controlRPT1");
static const wxString  KEY_CONTROL_RPT2       = wxT("/controlRPT2");
static const wxString  KEY_CONTROL_SHUTDOWN   = wxT("/controlShutdown");
static const wxString  KEY_CONTROL_STARTUP    = wxT("/controlStartup");
static const wxString  KEY_CONTROL_COMMAND1      = wxT("/controlCommand1");
static const wxString  KEY_CONTROL_COMMAND1_LINE = wxT("/controlCommand1Line");
static const wxString  KEY_CONTROL_COMMAND2      = wxT("/controlCommand2");
static const wxString  KEY_CONTROL_COMMAND2_LINE = wxT("/controlCommand2Line");
static const wxString  KEY_CONTROL_OUTPUT1    = wxT("/controlOutput1");
static const wxString  KEY_CONTROL_OUTPUT2    = wxT("/controlOutput2");
static const wxString  KEY_CONTROL_OUTPUT3    = wxT("/controlOutput3");
static const wxString  KEY_CONTROL_OUTPUT4    = wxT("/controlOutput4");
static const wxString  KEY_OUTPUT_1           = wxT("/output1");
static const wxString  KEY_OUTPUT_2           = wxT("/output2");
static const wxString  KEY_OUTPUT_3           = wxT("/output3");
static const wxString  KEY_OUTPUT_4           = wxT("/output4");
static const wxString  KEY_ACTIVE_HANG_TIME   = wxT("/activeHangTime");

static const wxString   DEFAULT_CALLSIGN           = wxT("GB3IN  C");
static const wxString   DEFAULT_GATEWAY            = wxEmptyString;
static const DSTAR_MODE DEFAULT_MODE               = MODE_DUPLEX;
static const bool       DEFAULT_ACK                = true;
static const wxString   DEFAULT_GATEWAY_ADDRESS    = wxEmptyString;
static const long       DEFAULT_GATEWAY_PORT       = 20000L;
static const wxString   DEFAULT_LOCAL_ADDRESS      = wxEmptyString;
static const long       DEFAULT_LOCAL_PORT         = 20000L;
static const long       DEFAULT_NETWORK_DELAY      = 0L;
static const wxString   DEFAULT_SOUND_READ_DEVICE  = wxEmptyString;
static const wxString   DEFAULT_SOUND_WRITE_DEVICE = wxEmptyString;
static const double     DEFAULT_LEVEL              = 1.0;
static const long       DEFAULT_PTT_DELAY          = 5L;
static const bool       DEFAULT_PTT_INVERT         = false;
static const bool       DEFAULT_GMSK_RXINV         = false;
static const bool       DEFAULT_GMSK_TXINV         = false;
static const wxString   DEFAULT_IFACE_TYPE         = wxEmptyString;
static const long       DEFAULT_TIMEOUT            = 180L;
static const long       DEFAULT_ACK_TIME           = 500L;
static const bool       DEFAULT_CONTROL_ENABLED    = false;
static const wxString   DEFAULT_CONTROL_RPT1       = wxEmptyString;
static const wxString   DEFAULT_CONTROL_RPT2       = wxEmptyString;
static const wxString   DEFAULT_CONTROL_SHUTDOWN   = wxEmptyString;
static const wxString   DEFAULT_CONTROL_STARTUP    = wxEmptyString;
static const wxString   DEFAULT_CONTROL_COMMAND1      = wxEmptyString;
static const wxString   DEFAULT_CONTROL_COMMAND1_LINE = wxEmptyString;
static const wxString   DEFAULT_CONTROL_COMMAND2      = wxEmptyString;
static const wxString   DEFAULT_CONTROL_COMMAND2_LINE = wxEmptyString;
static const wxString   DEFAULT_CONTROL_OUTPUT1    = wxEmptyString;
static const wxString   DEFAULT_CONTROL_OUTPUT2    = wxEmptyString;
static const wxString   DEFAULT_CONTROL_OUTPUT3    = wxEmptyString;
static const wxString   DEFAULT_CONTROL_OUTPUT4    = wxEmptyString;
static const bool       DEFAULT_OUTPUT_1           = false;
static const bool       DEFAULT_OUTPUT_2           = false;
static const bool       DEFAULT_OUTPUT_3           = false;
static const bool       DEFAULT_OUTPUT_4           = false;
static const long       DEFAULT_ACTIVE_HANG_TIME   = 0L;


CDStarRepeaterConfig::CDStarRepeaterConfig(wxConfigBase* config, const wxString& name) :
m_config(config),
m_name()
{
	wxASSERT(config != NULL);

	if (!name.IsEmpty())
		m_name = wxT("/") + name;
}

CDStarRepeaterConfig::~CDStarRepeaterConfig()
{
	delete m_config;
}

void CDStarRepeaterConfig::getCallsign(wxString& callsign, wxString& gateway, DSTAR_MODE& mode, bool& ack) const
{
	wxString callsignKey = m_name + KEY_CALLSIGN;
	wxString gatewayKey  = m_name + KEY_GATEWAY;
	wxString modeKey     = m_name + KEY_MODE;
	wxString ackKey      = m_name + KEY_ACK;

	m_config->Read(callsignKey, &callsign, DEFAULT_CALLSIGN);
	m_config->Read(gatewayKey,  &gateway,  DEFAULT_GATEWAY);

	long temp;
	m_config->Read(modeKey,     &temp,     long(DEFAULT_MODE));
	mode = DSTAR_MODE(temp);

	m_config->Read(ackKey,      &ack,      DEFAULT_ACK);
}

void CDStarRepeaterConfig::setCallsign(const wxString& callsign, const wxString& gateway, DSTAR_MODE mode, bool ack) const
{
	wxString callsignKey = m_name + KEY_CALLSIGN;
	wxString gatewayKey  = m_name + KEY_GATEWAY;
	wxString modeKey     = m_name + KEY_MODE;
	wxString ackKey      = m_name + KEY_ACK;

	m_config->Write(callsignKey, callsign);
	m_config->Write(gatewayKey,  gateway);
	m_config->Write(modeKey,     long(mode));
	m_config->Write(ackKey,      ack);
	m_config->Flush();
}

void CDStarRepeaterConfig::getNetwork(wxString& gatewayAddress, unsigned int& gatewayPort, wxString& localAddress, unsigned int& localPort, unsigned int& networkDelay) const
{
	wxString gatewayAddressKey = m_name + KEY_GATEWAY_ADDRESS;
	wxString gatewayPortKey    = m_name + KEY_GATEWAY_PORT;
	wxString localAddressKey   = m_name + KEY_LOCAL_ADDRESS;
	wxString localPortKey      = m_name + KEY_LOCAL_PORT;
	wxString networkDelayKey   = m_name + KEY_NETWORK_DELAY;

	m_config->Read(gatewayAddressKey,  &gatewayAddress,  DEFAULT_GATEWAY_ADDRESS);

	long temp;
	m_config->Read(gatewayPortKey,     &temp,            DEFAULT_GATEWAY_PORT);
	gatewayPort = (unsigned int)temp;

	m_config->Read(localAddressKey,    &localAddress,    DEFAULT_LOCAL_ADDRESS);

	m_config->Read(localPortKey,       &temp,            DEFAULT_LOCAL_PORT);
	localPort = (unsigned int)temp;

	m_config->Read(networkDelayKey,    &temp,            DEFAULT_NETWORK_DELAY);
	networkDelay = (unsigned int)temp;
}

void CDStarRepeaterConfig::setNetwork(const wxString& gatewayAddress, unsigned int gatewayPort, const wxString& localAddress, unsigned int localPort, unsigned int networkDelay) const
{
	wxString gatewayAddressKey = m_name + KEY_GATEWAY_ADDRESS;
	wxString gatewayPortKey    = m_name + KEY_GATEWAY_PORT;
	wxString localAddressKey   = m_name + KEY_LOCAL_ADDRESS;
	wxString localPortKey      = m_name + KEY_LOCAL_PORT;
	wxString networkDelayKey   = m_name + KEY_NETWORK_DELAY;

	m_config->Write(gatewayAddressKey, gatewayAddress);
	m_config->Write(gatewayPortKey,    long(gatewayPort));
	m_config->Write(localAddressKey,   localAddress);
	m_config->Write(localPortKey,      long(localPort));
	m_config->Write(networkDelayKey,   long(networkDelay));
	m_config->Flush();
}

void CDStarRepeaterConfig::getRadio(wxString& readDevice, wxString& writeDevice, wxFloat32& level, bool& rxInvert, bool& txInvert) const
{
	wxString readDeviceKey  = m_name + KEY_SOUND_READ_DEVICE;
	wxString writeDeviceKey = m_name + KEY_SOUND_WRITE_DEVICE;
	wxString levelKey       = m_name + KEY_LEVEL;
	wxString rxInvKey       = m_name + KEY_GMSK_RXINV;
	wxString txInvKey       = m_name + KEY_GMSK_TXINV;

	m_config->Read(readDeviceKey,  &readDevice,  DEFAULT_SOUND_READ_DEVICE);
	m_config->Read(writeDeviceKey, &writeDevice, DEFAULT_SOUND_WRITE_DEVICE);

	double temp1;
	m_config->Read(levelKey,       &temp1,       DEFAULT_LEVEL);
	level = wxFloat32(temp1);

	m_config->Read(rxInvKey,       &rxInvert,    DEFAULT_GMSK_RXINV);
	m_config->Read(txInvKey,       &txInvert,    DEFAULT_GMSK_TXINV);
}

void CDStarRepeaterConfig::setRadio(const wxString& readDevice, const wxString& writeDevice, wxFloat32 level, bool rxInvert, bool txInvert) const
{
	wxString readDeviceKey  = m_name + KEY_SOUND_READ_DEVICE;
	wxString writeDeviceKey = m_name + KEY_SOUND_WRITE_DEVICE;
	wxString levelKey       = m_name + KEY_LEVEL;
	wxString rxInvKey       = m_name + KEY_GMSK_RXINV;
	wxString txInvKey       = m_name + KEY_GMSK_TXINV;

	m_config->Write(readDeviceKey,  readDevice);
	m_config->Write(writeDeviceKey, writeDevice);
	m_config->Write(levelKey,       double(level));
	m_config->Write(rxInvKey,       rxInvert);
	m_config->Write(txInvKey,       txInvert);
	m_config->Flush();
}

void CDStarRepeaterConfig::getController(wxString& type, int& pttDelay, bool& pttInvert) const
{
	wxString ifaceTypeKey = m_name + KEY_IFACE_TYPE;
	wxString pttDelayKey  = m_name + KEY_PTT_DELAY;
	wxString pttInvertKey = m_name + KEY_PTT_INVERT;

	m_config->Read(ifaceTypeKey,     &type,          DEFAULT_IFACE_TYPE);

	long temp;
	m_config->Read(pttDelayKey,      &temp,          DEFAULT_PTT_DELAY);
	pttDelay = int(temp);

	m_config->Read(pttInvertKey,     &pttInvert,     DEFAULT_PTT_INVERT);
}

void CDStarRepeaterConfig::setController(const wxString& type, int pttDelay, bool pttInvert) const
{
	wxString ifaceTypeKey = m_name + KEY_IFACE_TYPE;
	wxString pttDelayKey  = m_name + KEY_PTT_DELAY;
	wxString pttInvertKey = m_name + KEY_PTT_INVERT;

	m_config->Write(ifaceTypeKey,     type);
	m_config->Write(pttDelayKey,      long(pttDelay));
	m_config->Write(pttInvertKey,     pttInvert);
	m_config->Flush();
}

void CDStarRepeaterConfig::getTimes(unsigned int& timeout, unsigned int& ackTime) const
{
	wxString timeoutKey = m_name + KEY_TIMEOUT;
	wxString ackTimeKey = m_name + KEY_ACK_TIME;

	long temp;
	m_config->Read(timeoutKey, &temp, DEFAULT_TIMEOUT);
	timeout = (unsigned int)temp;

	m_config->Read(ackTimeKey, &temp, DEFAULT_ACK_TIME);
	ackTime = (unsigned int)temp;
}

void CDStarRepeaterConfig::setTimes(unsigned int timeout, unsigned int ackTime) const
{
	wxString timeoutKey = m_name + KEY_TIMEOUT;
	wxString ackTimeKey = m_name + KEY_ACK_TIME;

	m_config->Write(timeoutKey, long(timeout));
	m_config->Write(ackTimeKey, long(ackTime));
	m_config->Flush();
}

void CDStarRepeaterConfig::getControl(bool& enabled, wxString& rpt1Callsign, wxString& rpt2Callsign, wxString& shutdown, wxString& startup, wxString& command1, wxString& command1Line, wxString& command2, wxString& command2Line, wxString& output1, wxString& output2, wxString& output3, wxString& output4) const
{
	wxString enabledKey      = m_name + KEY_CONTROL_ENABLED;
	wxString rpt1CallsignKey = m_name + KEY_CONTROL_RPT1;
	wxString rpt2CallsignKey = m_name + KEY_CONTROL_RPT2;
	wxString shutdownKey     = m_name + KEY_CONTROL_SHUTDOWN;
	wxString startupKey      = m_name + KEY_CONTROL_STARTUP;
	wxString command1Key     = m_name + KEY_CONTROL_COMMAND1;
	wxString command1LineKey = m_name + KEY_CONTROL_COMMAND1_LINE;
	wxString command2Key     = m_name + KEY_CONTROL_COMMAND2;
	wxString command2LineKey = m_name + KEY_CONTROL_COMMAND2_LINE;
	wxString output1Key      = m_name + KEY_CONTROL_OUTPUT1;
	wxString output2Key      = m_name + KEY_CONTROL_OUTPUT2;
	wxString output3Key      = m_name + KEY_CONTROL_OUTPUT3;
	wxString output4Key      = m_name + KEY_CONTROL_OUTPUT4;

	m_config->Read(enabledKey,      &enabled,      DEFAULT_CONTROL_ENABLED);
	m_config->Read(rpt1CallsignKey, &rpt1Callsign, DEFAULT_CONTROL_RPT1);
	m_config->Read(rpt2CallsignKey, &rpt2Callsign, DEFAULT_CONTROL_RPT2);
	m_config->Read(shutdownKey,     &shutdown,     DEFAULT_CONTROL_SHUTDOWN);
	m_config->Read(startupKey,      &startup,      DEFAULT_CONTROL_STARTUP);
	m_config->Read(command1Key,     &command1,     DEFAULT_CONTROL_COMMAND1);
	m_config->Read(command1LineKey, &command1Line, DEFAULT_CONTROL_COMMAND1_LINE);
	m_config->Read(command2Key,     &command2,     DEFAULT_CONTROL_COMMAND2);
	m_config->Read(command2LineKey, &command2Line, DEFAULT_CONTROL_COMMAND2_LINE);
	m_config->Read(output1Key,      &output1,      DEFAULT_CONTROL_OUTPUT1);
	m_config->Read(output2Key,      &output2,      DEFAULT_CONTROL_OUTPUT2);
	m_config->Read(output3Key,      &output3,      DEFAULT_CONTROL_OUTPUT3);
	m_config->Read(output4Key,      &output4,      DEFAULT_CONTROL_OUTPUT4);
}

void CDStarRepeaterConfig::setControl(bool enabled, const wxString& rpt1Callsign, const wxString& rpt2Callsign, const wxString& shutdown, const wxString& startup, const wxString& command1, const wxString& command1Line, const wxString& command2, const wxString& command2Line, const wxString& output1, const wxString& output2, const wxString& output3, const wxString& output4) const
{
	wxString enabledKey      = m_name + KEY_CONTROL_ENABLED;
	wxString rpt1CallsignKey = m_name + KEY_CONTROL_RPT1;
	wxString rpt2CallsignKey = m_name + KEY_CONTROL_RPT2;
	wxString shutdownKey     = m_name + KEY_CONTROL_SHUTDOWN;
	wxString startupKey      = m_name + KEY_CONTROL_STARTUP;
	wxString command1Key     = m_name + KEY_CONTROL_COMMAND1;
	wxString command1LineKey = m_name + KEY_CONTROL_COMMAND1_LINE;
	wxString command2Key     = m_name + KEY_CONTROL_COMMAND2;
	wxString command2LineKey = m_name + KEY_CONTROL_COMMAND2_LINE;
	wxString output1Key      = m_name + KEY_CONTROL_OUTPUT1;
	wxString output2Key      = m_name + KEY_CONTROL_OUTPUT2;
	wxString output3Key      = m_name + KEY_CONTROL_OUTPUT3;
	wxString output4Key      = m_name + KEY_CONTROL_OUTPUT4;

	m_config->Write(enabledKey,      enabled);
	m_config->Write(rpt1CallsignKey, rpt1Callsign);
	m_config->Write(rpt2CallsignKey, rpt2Callsign);
	m_config->Write(shutdownKey,     shutdown);
	m_config->Write(startupKey,      startup);
	m_config->Write(command1Key,     command1);
	m_config->Write(command1LineKey, command1Line);
	m_config->Write(command2Key,     command2);
	m_config->Write(command2LineKey, command2Line);
	m_config->Write(output1Key,      output1);
	m_config->Write(output2Key,      output2);
	m_config->Write(output3Key,      output3);
	m_config->Write(output4Key,      output4);
	m_config->Flush();
}

void CDStarRepeaterConfig::getOutputs(bool& out1, bool& out2, bool& out3, bool& out4) const
{
	wxString output1Key = m_name + KEY_OUTPUT_1;
	wxString output2Key = m_name + KEY_OUTPUT_2;
	wxString output3Key = m_name + KEY_OUTPUT_3;
	wxString output4Key = m_name + KEY_OUTPUT_4;

	m_config->Read(output1Key, &out1, DEFAULT_OUTPUT_1);
	m_config->Read(output2Key, &out2, DEFAULT_OUTPUT_2);
	m_config->Read(output3Key, &out3, DEFAULT_OUTPUT_3);
	m_config->Read(output4Key, &out4, DEFAULT_OUTPUT_4);
}

void CDStarRepeaterConfig::setOutputs(bool out1, bool out2, bool out3, bool out4) const
{
	wxString output1Key = m_name + KEY_OUTPUT_1;
	wxString output2Key = m_name + KEY_OUTPUT_2;
	wxString output3Key = m_name + KEY_OUTPUT_3;
	wxString output4Key = m_name + KEY_OUTPUT_4;

	m_config->Write(output1Key, out1);
	m_config->Write(output2Key, out2);
	m_config->Write(output3Key, out3);
	m_config->Write(output4Key, out4);
	m_config->Flush();
}

void CDStarRepeaterConfig::getActiveHang(unsigned int& time) const
{
	wxString timeKey = m_name + KEY_ACTIVE_HANG_TIME;

	long temp;
	m_config->Read(timeKey, &temp, DEFAULT_ACTIVE_HANG_TIME);
	time = (unsigned int)temp;
}

void CDStarRepeaterConfig::setActiveHang(unsigned int time) const
{
	wxString timeKey = m_name + KEY_ACTIVE_HANG_TIME;

	m_config->Write(timeKey, long(time));
	m_config->Flush();
}
