/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	DStarRepeaterThread_H
#define	DStarRepeaterThread_H

#include "DStarRepeaterStatusData.h"
#include "SoundCardReaderWriter.h"
#include "DStarGMSKDemodulator.h"
#include "DStarRepeaterAMBEFEC.h"
#include "LimitedLengthBuffer.h"
#include "DStarGMSKModulator.h"
#include "ExternalController.h"
#include "DStarRepeaterDefs.h"
#include "SlowDataDecoder.h"
#include "ProtocolHandler.h"
#include "PatternMatcher.h"
#include "WAVFileReader.h"
#include "AudioCallback.h"
#include "WAVFileWriter.h"
#include "NetworkDelay.h"
#include "HeaderData.h"
#include "AudioDelay.h"
#include "RingBuffer.h"
#include "PTTDelay.h"
#include "Timer.h"

#include <wx/wx.h>

enum FRAME_TYPE {
	FRAME_NORMAL,
	FRAME_SYNC,
	FRAME_END
};

class CDStarRepeaterThread : public wxThread, public IAudioCallback {
public:
	CDStarRepeaterThread();
	virtual ~CDStarRepeaterThread();

	virtual void callback(const wxFloat32* input, wxFloat32* output, unsigned int nSamples, int id);

	virtual void setCallsign(const wxString& callsign, const wxString& gateway, DSTAR_MODE mode, bool ack);
	virtual void setProtocolHandler(IProtocolHandler* handler, unsigned int networkDelay);
	virtual void setSoundCard(CSoundCardReaderWriter* soundcard, wxFloat32 level, bool rxInvert, bool txInvert);
	virtual void setController(CExternalController* controller, int pttDelay);
	virtual void setTimes(unsigned int timeout, unsigned int ackTime);
	virtual void setControl(bool enabled, const wxString& rpt1Callsign, const wxString& rpt2Callsign, const wxString& shutdown, const wxString& startup, const wxString& command1, const wxString& command1Line, const wxString& command2, const wxString& command2Line, const wxString& output1, const wxString& output2, const wxString& output3, const wxString& output4);
	virtual void setActiveHang(unsigned int time);
	virtual void setOutputs(bool out1, bool out2, bool out3, bool out4);
#if defined(__WXDEBUG__)
	virtual void setReader(CWAVFileReader* reader);
#endif

	virtual void shutdown();
	virtual void startup();

	virtual void command1();
	virtual void command2();

	virtual CDStarRepeaterStatusData* getStatus();

	virtual void* Entry();
	virtual void  kill();

	virtual bool  init();
	virtual void  finish();

	virtual bool  isActive() const;
	virtual void  reset();
	virtual void  clock();

private:
	CSoundCardReaderWriter* m_soundcard;
	IProtocolHandler*       m_protocolHandler;
	CNetworkDelay*          m_networkDelay;
	CExternalController*    m_controller;
	CPTTDelay*              m_pttDelay;
	CAudioDelay*            m_audioDelay;
	bool                    m_stopped;
	CRingBuffer<wxFloat32>  m_inBuffer;
	CRingBuffer<wxFloat32>  m_outBuffer;
	CRingBuffer<bool>       m_dataBuffer;
	wxString                m_rptCallsign;
	wxString                m_gwyCallsign;
	CWAVFileReader*         m_reader;
	CDStarGMSKDemodulator   m_demodulator;
	CDStarGMSKModulator     m_modulator;
	CLimitedLengthBuffer    m_bitBuffer;
	DSTAR_RX_STATE          m_rxState;
	CPatternMatcher         m_frameMatcher;
	CPatternMatcher         m_dataMatcher;
	CPatternMatcher         m_endMatcher;
	CWAVFileWriter*         m_writer;
	CHeaderData*            m_userHeader;
	CHeaderData*            m_statusHeader;
	unsigned int            m_radioSyncCounter;
	unsigned int            m_radioSyncsLost;
	unsigned int            m_networkSyncCounter;
	CTimer                  m_timeoutTimer;
	CTimer                  m_watchdogTimer;
	CTimer                  m_pollTimer;
	CTimer                  m_ackTimer;
	DSTAR_RPT_STATE         m_rptState;
	CSlowDataDecoder        m_slowData;
	bool                    m_tx;
	bool                    m_killed;
	DSTAR_MODE              m_mode;
	bool                    m_ack;
	wxFloat32               m_level;
	bool                    m_controlEnabled;
	wxString                m_controlRPT1;
	wxString                m_controlRPT2;
	wxString                m_controlShutdown;
	wxString                m_controlStartup;
	wxString                m_controlCommand1;
	wxString                m_controlCommand1Line;
	wxString                m_controlCommand2;
	wxString                m_controlCommand2Line;
	wxString                m_controlOutput1;
	wxString                m_controlOutput2;
	wxString                m_controlOutput3;
	wxString                m_controlOutput4;
	bool                    m_output1;
	bool                    m_output2;
	bool                    m_output3;
	bool                    m_output4;
	CTimer                  m_activeHangTimer;
	bool                    m_shutdown;
	bool                    m_disable;
	CDStarRepeaterAMBEFEC   m_ambe;
	unsigned int            m_ambeBits;
	unsigned int            m_ambeErrors;

	void receive();
	void transmitHeader(CHeaderData& header);
	void transmitStatus();
	void transmitErrorStatus();
	void transmit();
	void transmitBits(const bool* bits, unsigned int length);

	void radioStateMachine(bool bit);
	void repeaterStateMachine();
	bool processRadioHeader(CHeaderData* header);
	void processNetworkHeader(NETWORK_TYPE type, CHeaderData* header);
	void processRadioFrame(FRAME_TYPE type);
	void processNetworkFrame(NETWORK_TYPE type, unsigned char* data, unsigned int length, bool end);
	void endOfRadioData();
	CHeaderData* processFECHeader();
	CHeaderData* processSlowData(bool sync);
	void setRadioState(DSTAR_RX_STATE state);
	bool setRepeaterState(DSTAR_RPT_STATE state);
	void readRadioData(const wxFloat32* buffer, unsigned int nSamples);
	void processNetwork(NETWORK_TYPE type, CHeaderData* header);
	void processNetwork(NETWORK_TYPE type, unsigned char* buffer, unsigned int length, bool end);
	bool checkControl(const CHeaderData& header);
};

#endif
